from .db import DB
from os.path import join
from .config import base_folder
from .tools import (
    create_folder,
    get_tables,
    create_spark_session,
    server_db,
    arquee_error,
)
from .logger import logger
from py4j.protocol import Py4JJavaError, Py4JError

import vaex
from vaex.dataframe import DataFrameLocal
import signal


class Arquee:
    def __init__(
        self,
        origin_db: str = "ORIGIN_DB",
        destiny_db: str = "DESTINY_DB",
        tables_file: str = "tables.txt",
    ):
        server_db.create_row()
        self.origin_db = origin_db
        self.destiny_db = destiny_db
        self.datalake = join(base_folder, "data", "datalake")
        self.warehouse = join(base_folder, "data", "warehouse")
        self.tables_file = tables_file
        create_folder(self.warehouse)

        self.spark = None

        signal.signal(signal.SIGINT, self.handle_interrupt)

    def handle_interrupt(self):
        print("\nInterrupción detectada (Ctrl + C). Cerrando correctamente...")
        server_db.failed_row()
        exit(0)

    def create_datalake(self):
        create_folder(self.datalake)

        self.spark = create_spark_session()
        self.tables = get_tables(self.tables_file)
        self.origin_db = DB(self.origin_db)
        self.destiny_db = DB(self.destiny_db)

        if self.origin_db.url == self.destiny_db.url:
            arquee_error(
                f"{self.origin_db} and {self.destiny_db} are the same database"
            )

        for table in self.tables:
            try:
                table_path = join(self.datalake, table)
                df = self.spark.read.jdbc(self.origin_db.url, table)
                df.write.parquet(table_path, mode="overwrite")
                logger.info(f"Table {table} loaded successfully on datalake")
            except Py4JJavaError as e:
                logger.error(
                    f"{table} does not exist in the database {self.origin_db.db_name}"
                )
                logger.error(e)
                continue
            except Py4JError as e:
                logger.error(e)
                self.handle_interrupt()

    def get_dataframe(self, table_name: str) -> DataFrameLocal:
        table_path = join(self.datalake, table_name, "*.parquet")
        return vaex.open(table_path)

    def close(self):
        if self.spark:
            self.spark.stop()
            logger.info("Spark session closed")
        server_db.done_row()
        logger.info("Arquee instance closed properly")

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.close()
